﻿//------------------------------------------------------------------------------
// QueryStringPagerField.cs
//
// QueryStringPagerField is a sample PagerField that uses a QueryString
// field to get the current page index. It renders links in the pager, but
// users can also navigate by changing the value of the query string field
// directly.
//
//------------------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.Drawing.Design;
using System.Globalization;
using System.Text;
using System.Web;
using System.Web.Resources;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace PHuff.WebControls {
    public class QueryStringPagerField : NumericPagerField {
        private const string QueryStringField = "CurrentPageIndex";
        public QueryStringPagerField() {
        }

        protected override DataPagerField CreateField() {
            return new QueryStringPagerField();
        }
        
        private HyperLink CreateNumericButton(int pageIndex) {
            HyperLink link = new HyperLink();
            StringBuilder sb = new StringBuilder();
            
            sb.Append(DataPager.Page.Request.AppRelativeCurrentExecutionFilePath);
            sb.Append("?");
            sb.Append(QueryStringField);
            sb.Append("=");
            sb.Append(pageIndex.ToString(CultureInfo.InvariantCulture));

            link.Text = (pageIndex + 1).ToString(CultureInfo.InvariantCulture);
            link.NavigateUrl = sb.ToString();

            if (!String.IsNullOrEmpty(NumericButtonCssClass)) {
                link.CssClass = NumericButtonCssClass;
            }

            return link;
        }

        private HyperLink CreateNextPrevButton(string buttonText, int pageIndex, string imageUrl) {
            HyperLink link = new HyperLink();
            StringBuilder sb = new StringBuilder();
            
            sb.Append(DataPager.Page.Request.AppRelativeCurrentExecutionFilePath);
            sb.Append("?");
            sb.Append(QueryStringField);
            sb.Append("=");
            sb.Append(pageIndex.ToString(CultureInfo.InvariantCulture));

            link.Text = buttonText;
            link.NavigateUrl = sb.ToString();
            link.ImageUrl = imageUrl;

            WebControl webControl = link as WebControl;
            if (!String.IsNullOrEmpty(NextPreviousButtonCssClass)) {
                link.CssClass = NextPreviousButtonCssClass;
            }

            return link;
        }

        public override void CreateDataPagers(DataPagerFieldItem container, int startRowIndex, int maximumRows, int totalRowCount, int fieldIndex) {            

            int currentPageIndex = startRowIndex / DataPager.PageSize;
            object currentPageObj = DataPager.Page.Request.QueryString[QueryStringField];
            short currentQSPageIndex;
            bool resetProperties = false;
            if (currentPageObj != null) {
                bool parsed = Int16.TryParse(currentPageObj.ToString(), out currentQSPageIndex);
                if (parsed) {
                    int highestPageIndex = (totalRowCount - 1)/maximumRows;
                    if (currentPageIndex != currentQSPageIndex && currentQSPageIndex <= highestPageIndex) {
                        currentPageIndex = currentQSPageIndex;
                        startRowIndex = (currentPageIndex * DataPager.PageSize);
                        resetProperties = true;
                    }
                }
            }

            int firstButtonIndex = (startRowIndex / (ButtonCount * DataPager.PageSize)) * ButtonCount;
            int lastButtonIndex = firstButtonIndex + ButtonCount - 1;
            int lastRecordIndex = ((lastButtonIndex + 1) * DataPager.PageSize) - 1;

            if (firstButtonIndex != 0) {
                container.Controls.Add(CreateNextPrevButton(PreviousPageText, firstButtonIndex - 1, PreviousPageImageUrl));
                if (RenderNonBreakingSpacesBetweenControls) {
                    container.Controls.Add(new LiteralControl("&nbsp;"));
                }
            }

            for (int i = 0; i < ButtonCount && totalRowCount > ((i + firstButtonIndex) * DataPager.PageSize); i++) {
                if (i + firstButtonIndex == currentPageIndex) {
                    Label pageNumber = new Label();
                    pageNumber.Text = (i + firstButtonIndex + 1).ToString(CultureInfo.InvariantCulture);
                    if (!String.IsNullOrEmpty(CurrentPageLabelCssClass)) {
                        pageNumber.CssClass = CurrentPageLabelCssClass;
                    }
                    container.Controls.Add(pageNumber);
                }
                else {
                    container.Controls.Add(CreateNumericButton(i + firstButtonIndex));
                }
                if (RenderNonBreakingSpacesBetweenControls) {
                    container.Controls.Add(new LiteralControl("&nbsp;"));
                }
            }

            if (lastRecordIndex < totalRowCount - 1) {
                if (RenderNonBreakingSpacesBetweenControls) {
                    container.Controls.Add(new LiteralControl("&nbsp;"));
                }
                container.Controls.Add(CreateNextPrevButton(NextPageText, firstButtonIndex + ButtonCount, NextPageImageUrl));
                if (RenderNonBreakingSpacesBetweenControls) {
                    container.Controls.Add(new LiteralControl("&nbsp;"));
                }
            }
            
            if (resetProperties) {
                DataPager.SetPageProperties(startRowIndex, maximumRows, true);
            }
        }

        public override void HandleEvent(CommandEventArgs e) {
            // Not needed because Hyperlinks don't generate bubbled events.
        }

        // Required for design-time support (DesignerPagerStyle)
        public override bool Equals(object o) {
            QueryStringPagerField field = o as QueryStringPagerField;
            if (field != null) {
                return base.Equals(o);
            }
            return false;
        }

        public override int GetHashCode() {
            return base.GetHashCode();
        }
    }
}
